@@ -63,7 +63,6 @@ main (void)
   gnutls_anon_client_credentials_t anoncred;
   gnutls_ia_client_credentials_t iacred;
   /* Need to enable anonymous KX specifically. */
-  const int kx_prio[] = { GNUTLS_KX_ANON_DH, 0 };
 
   gnutls_global_init ();
 
@@ -79,8 +78,7 @@ main (void)
   gnutls_init (&session, GNUTLS_CLIENT);
 
   /* Use default priorities */
-  gnutls_set_default_priority2 (session, GNUTLS_PRIORITIES_SECURITY_NORMAL);
-  gnutls_kx_set_priority (session, kx_prio);
+  gnutls_priority_set_direct (session, "NORMAL:+ANON-DH", NULL);
 
   /* put the anonymous and TLS/IA credentials to the current session
    */
