@@ -890,12 +890,13 @@ gnutls_x509_crt_get_pk_algorithm (gnutls_x509_crt_t cert, unsigned int *bits)
 
 #define XMPP_OID "1.3.6.1.5.5.7.8.5"
 
-/* returns the type and the name.
+/* returns the type and the name on success.
+ * Type is also returned as a parameter in case of an error.
  */
 static int
 parse_general_name (ASN1_TYPE src, const char *src_name,
-		    int seq, void *name, size_t * name_size,
-		    int othername_oid)
+		    int seq, void *name, size_t * name_size, 
+		    unsigned int* ret_type, int othername_oid)
 {
   int len;
   char nptr[MAX_NAME_SIZE];
@@ -931,6 +932,9 @@ parse_general_name (ASN1_TYPE src, const char *src_name,
       gnutls_assert ();
       return GNUTLS_E_X509_UNKNOWN_SAN;
     }
+  
+  if (ret_type)
+    *ret_type = type;
 
   if (type == GNUTLS_SAN_OTHERNAME)
     {
@@ -1026,16 +1030,18 @@ parse_general_name (ASN1_TYPE src, const char *src_name,
 
       len = *name_size;
       result = asn1_read_value (src, nptr, name, &len);
-      *name_size = len;
+      *name_size = len + 1;
 
       if (result == ASN1_MEM_ERROR)
 	return GNUTLS_E_SHORT_MEMORY_BUFFER;
-
+      
       if (result != ASN1_SUCCESS)
 	{
 	  gnutls_assert ();
 	  return _gnutls_asn2err (result);
 	}
+      
+      ((char*)name)[len] = 0;
     }
 
   return type;
@@ -1044,7 +1050,7 @@ parse_general_name (ASN1_TYPE src, const char *src_name,
 static int
 get_subject_alt_name (gnutls_x509_crt_t cert,
 		      unsigned int seq, void *ret,
-		      size_t * ret_size,
+		      size_t * ret_size, unsigned int *ret_type,
 		      unsigned int *critical,
 		      int othername_oid)
 {
@@ -1096,7 +1102,7 @@ get_subject_alt_name (gnutls_x509_crt_t cert,
       return _gnutls_asn2err (result);
     }
 
-  result = parse_general_name (c2, "", seq, ret, ret_size, othername_oid);
+  result = parse_general_name (c2, "", seq, ret, ret_size, ret_type, othername_oid);
 
   asn1_delete_structure (&c2);
 
@@ -1142,6 +1148,7 @@ get_subject_alt_name (gnutls_x509_crt_t cert,
   * with the required size.  If the certificate does not have an
   * Alternative name with the specified sequence number then
   * %GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE is returned.
+  *
   **/
 int
 gnutls_x509_crt_get_subject_alt_name (gnutls_x509_crt_t cert,
@@ -1149,7 +1156,35 @@ gnutls_x509_crt_get_subject_alt_name (gnutls_x509_crt_t cert,
 				      size_t * ret_size,
 				      unsigned int *critical)
 {
-  return get_subject_alt_name (cert, seq, ret, ret_size, critical, 0);
+  return get_subject_alt_name (cert, seq, ret, ret_size, NULL, critical, 0);
+}
+
+/**
+  * gnutls_x509_crt_get_subject_alt_name2 - Get certificate's alternative name, if any
+  * @cert: should contain a gnutls_x509_crt_t structure
+  * @seq: specifies the sequence number of the alt name (0 for the first one, 1 for the second etc.)
+  * @ret: is the place where the alternative name will be copied to
+  * @ret_size: holds the size of ret.
+  * @ret_type: holds the type of the alternative name (one of gnutls_x509_subject_alt_name_t).
+  * @critical: will be non zero if the extension is marked as critical (may be null)
+  *
+  * This function will return the alternative names, contained in the
+  * given certificate. It is the same as gnutls_x509_crt_get_subject_alt_name()
+  * except for the fact that it will return the type of the alternative
+  * name in @ret_type even if the function fails for some reason (i.e.
+  * the buffer provided is not enough).
+  *
+  * The return values are the same as with gnutls_x509_crt_get_subject_alt_name().
+  *
+  **/
+int
+gnutls_x509_crt_get_subject_alt_name2 (gnutls_x509_crt_t cert,
+				      unsigned int seq, void *ret,
+				      size_t * ret_size, 
+                                      unsigned int* ret_type,
+				      unsigned int *critical)
+{
+  return get_subject_alt_name (cert, seq, ret, ret_size, ret_type, critical, 0);
 }
 
 /**
@@ -1184,7 +1219,7 @@ gnutls_x509_crt_get_subject_alt_othername_oid (gnutls_x509_crt_t cert,
 					       void *ret,
 					       size_t * ret_size)
 {
-  return get_subject_alt_name (cert, seq, ret, ret_size, NULL, 1);
+  return get_subject_alt_name (cert, seq, ret, ret_size, NULL, NULL, 1);
 }
 
 /**
@@ -1874,10 +1909,9 @@ gnutls_x509_dn_get_rdn_ava (gnutls_x509_dn_t dn,
   *
   * If the buffer is null then only the size will be filled.
   *
-  * Returns GNUTLS_E_SHORT_MEMORY_BUFFER if the provided buffer is not
-  * long enough, and in that case the *sizeof_buf will be updated with
-  * the required size.  On success 0 is returned.
-  *
+  * Returns: %GNUTLS_E_SHORT_MEMORY_BUFFER if the provided buffer is
+  * not long enough, and in that case the *sizeof_buf will be updated
+  * with the required size.  On success 0 is returned.
   **/
 int
 gnutls_x509_crt_get_fingerprint (gnutls_x509_crt_t cert,
@@ -1941,7 +1975,6 @@ gnutls_x509_crt_get_fingerprint (gnutls_x509_crt_t cert,
   *
   * Return value: In case of failure a negative value will be
   *   returned, and 0 on success.
-  *
   **/
 int
 gnutls_x509_crt_export (gnutls_x509_crt_t cert,
@@ -2046,7 +2079,6 @@ rsadsa_get_key_id (gnutls_x509_crt_t crt, int pk,
   *
   * Return value: In case of failure a negative value will be
   *   returned, and 0 on success.
-  *
   **/
 int
 gnutls_x509_crt_get_key_id (gnutls_x509_crt_t crt, unsigned int flags,
@@ -2128,12 +2160,11 @@ gnutls_x509_crt_get_key_id (gnutls_x509_crt_t crt, unsigned int flags,
   * @crl_list: should contain a list of gnutls_x509_crl_t structures
   * @crl_list_length: the length of the crl_list
   *
-  * This function will return check if the given certificate is revoked.
-  * It is assumed that the CRLs have been verified before.
-  *
-  * Returns 0 if the certificate is NOT revoked, and 1 if it is.
-  * A negative value is returned on error. 
+  * This function will return check if the given certificate is
+  * revoked.  It is assumed that the CRLs have been verified before.
   *
+  * Returns: 0 if the certificate is NOT revoked, and 1 if it is.  A
+  * negative value is returned on error.
   **/
 int
 gnutls_x509_crt_check_revocation (gnutls_x509_crt_t cert,
@@ -2237,12 +2268,11 @@ gnutls_x509_crt_check_revocation (gnutls_x509_crt_t cert,
   * @data: holds the data to be signed
   * @signature: contains the signature
   *
-  * This function will verify the given signed data, using the parameters from the
-  * certificate.
-  *
-  * In case of a verification failure 0 is returned, and
-  * 1 on success.
+  * This function will verify the given signed data, using the
+  * parameters from the certificate.
   *
+  * Returns: In case of a verification failure 0 is returned, and 1 on
+  * success.
   **/
 int
 gnutls_x509_crt_verify_data (gnutls_x509_crt_t crt, unsigned int flags,
@@ -2276,26 +2306,30 @@ gnutls_x509_crt_verify_data (gnutls_x509_crt_t crt, unsigned int flags,
   * @reason_flags: Revocation reasons flags.
   * @critical: will be non zero if the extension is marked as critical (may be null)
   *
-  * This function will return the CRL distribution points (2.5.29.31), contained in the
-  * given certificate.
+  * This function will return the CRL distribution points (2.5.29.31),
+  * contained in the given certificate.
   *
-  * @reason_flags should be an ORed sequence of GNUTLS_CRL_REASON_UNUSED,
-  * GNUTLS_CRL_REASON_KEY_COMPROMISE, GNUTLS_CRL_REASON_CA_COMPROMISE,
-  * GNUTLS_CRL_REASON_AFFILIATION_CHANGED, GNUTLS_CRL_REASON_SUPERSEEDED,
-  * GNUTLS_CRL_REASON_CESSATION_OF_OPERATION, GNUTLS_CRL_REASON_CERTIFICATE_HOLD,
-  * GNUTLS_CRL_REASON_PRIVILEGE_WITHDRAWN, GNUTLS_CRL_REASON_AA_COMPROMISE,
-  * or zero for all possible reasons.
-  * 
-  * This is specified in X509v3 Certificate Extensions. GNUTLS will return the 
-  * distribution point type, or a negative error code on error.
+  * @reason_flags should be an ORed sequence of
+  * GNUTLS_CRL_REASON_UNUSED, GNUTLS_CRL_REASON_KEY_COMPROMISE,
+  * GNUTLS_CRL_REASON_CA_COMPROMISE,
+  * GNUTLS_CRL_REASON_AFFILIATION_CHANGED,
+  * GNUTLS_CRL_REASON_SUPERSEEDED,
+  * GNUTLS_CRL_REASON_CESSATION_OF_OPERATION,
+  * GNUTLS_CRL_REASON_CERTIFICATE_HOLD,
+  * GNUTLS_CRL_REASON_PRIVILEGE_WITHDRAWN,
+  * GNUTLS_CRL_REASON_AA_COMPROMISE, or zero for all possible reasons.
   *
-  * Returns GNUTLS_E_SHORT_MEMORY_BUFFER and updates &ret_size if &ret_size is not enough to hold the distribution
-  * point, or the type of the distribution point if everything was ok. The type is 
-  * one of the enumerated gnutls_x509_subject_alt_name_t.
-  *
-  * If the certificate does not have an Alternative name with the specified 
-  * sequence number then returns GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
+  * This is specified in X509v3 Certificate Extensions. GNUTLS will
+  * return the distribution point type, or a negative error code on
+  * error.
   *
+  * Returns %GNUTLS_E_SHORT_MEMORY_BUFFER and updates &@ret_size if
+  * &@ret_size is not enough to hold the distribution point, or the
+  * type of the distribution point if everything was ok. The type is
+  * one of the enumerated %gnutls_x509_subject_alt_name_t.  If the
+  * certificate does not have an Alternative name with the specified
+  * sequence number then %GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE is
+  * returned.
   **/
 int
 gnutls_x509_crt_get_crl_dist_points (gnutls_x509_crt_t cert,
@@ -2364,7 +2398,7 @@ gnutls_x509_crt_get_crl_dist_points (gnutls_x509_crt_t cert,
    */
   _gnutls_str_cpy (name, sizeof (name), "?1.distributionPoint.fullName");
 
-  result = parse_general_name (c2, name, seq, ret, ret_size, 0);
+  result = parse_general_name (c2, name, seq, ret, ret_size, NULL, 0);
   if (result < 0)
     {
       asn1_delete_structure (&c2);
@@ -2412,10 +2446,9 @@ gnutls_x509_crt_get_crl_dist_points (gnutls_x509_crt_t cert,
   *
   * If @oid is null then only the size will be filled.
   *
-  * Returns GNUTLS_E_SHORT_MEMORY_BUFFER if the provided buffer is not
-  * long enough, and in that case the *sizeof_oid will be updated with
-  * the required size.  On success 0 is returned.
-  *
+  * Returns: %GNUTLS_E_SHORT_MEMORY_BUFFER if the provided buffer is
+  * not long enough, and in that case the *sizeof_oid will be updated
+  * with the required size.  On success 0 is returned.
   **/
 int
 gnutls_x509_crt_get_key_purpose_oid (gnutls_x509_crt_t cert,
@@ -2502,10 +2535,11 @@ gnutls_x509_crt_get_key_purpose_oid (gnutls_x509_crt_t cert,
   * @m: will hold the modulus
   * @e: will hold the public exponent
   *
-  * This function will export the RSA private key's parameters found in the given
-  * structure. The new parameters will be allocated using
+  * This function will export the RSA public key's parameters found in
+  * the given structure.  The new parameters will be allocated using
   * gnutls_malloc() and will be stored in the appropriate datum.
-  * 
+  *
+  * Returns: %GNUTLS_E_SUCCESS on success, otherwise an error.
   **/
 int
 gnutls_x509_crt_get_pk_rsa_raw (gnutls_x509_crt_t crt,
@@ -2562,17 +2596,18 @@ cleanup:
 }
 
 /**
-  * gnutls_x509_crt_get_pk_dsa_raw - This function will export the DSA private key
+  * gnutls_x509_crt_get_pk_dsa_raw - This function will export the DSA public key
   * @crt: Holds the certificate
   * @p: will hold the p
   * @q: will hold the q
   * @g: will hold the g
   * @y: will hold the y
   *
-  * This function will export the DSA private key's parameters found in the given
-  * certificate. The new parameters will be allocated using
+  * This function will export the DSA public key's parameters found in
+  * the given certificate.  The new parameters will be allocated using
   * gnutls_malloc() and will be stored in the appropriate datum.
-  * 
+  *
+  * Returns: %GNUTLS_E_SUCCESS on success, otherwise an error.
   **/
 int
 gnutls_x509_crt_get_pk_dsa_raw (gnutls_x509_crt_t crt,
@@ -2667,14 +2702,13 @@ cleanup:
   * @flags: must be zero or an OR'd sequence of gnutls_certificate_import_flags.
   *
   * This function will convert the given PEM encoded certificate list
-  * to the native gnutls_x509_crt_t format. The output will be stored in @certs.
-  * They will be automatically initialized.
+  * to the native gnutls_x509_crt_t format. The output will be stored
+  * in @certs.  They will be automatically initialized.
   *
-  * If the Certificate is PEM encoded it should have a header of "X509 CERTIFICATE", or
-  * "CERTIFICATE".
-  *
-  * Returns the number of certificates read or a negative error value.
+  * If the Certificate is PEM encoded it should have a header of "X509
+  * CERTIFICATE", or "CERTIFICATE".
   *
+  * Returns: the number of certificates read or a negative error value.
   **/
 int
 gnutls_x509_crt_list_import (gnutls_x509_crt_t * certs,
